"""Speks — Interactive Functional Analysis Generator."""

from speks.engine.mocking import (
    ExternalService,
    MockErrorResponse,
    MockResponse,
    ServiceError,
)

__all__ = ["ExternalService", "MockErrorResponse", "MockResponse", "ServiceError"]
__version__ = "0.1.1"
__author__ = "Angeall Software"
